#!/bin/bash

# **********************************************************************
# Agente Generico Pandora 
# Linux version 
# (c) Sancho Lerena 2003-2005, <slerena@gmail.com>
# Este codigo esta licenciado bajo la licencia GPL 2.0.
# This code is licenced under GPL 2.0 licence.
# **********************************************************************

IFS=$'\n'
AGENT_VERSION=1.1

# Begin cycle for adquire primary config tokens
TIMESTAMP=`date +"%Y/%m/%d %H:%M:%S"`

if [ -z "$1" ]
then
	echo " "
	echo "FATAL ERROR: I need an argument to PANDORA AGENT home path"
	echo " "
	echo " example:   /opt/pandora_ng/pandora_agent.sh /opt/pandora_ng  "
	echo " "
	exit -1
else
	PANDORA_HOME=$1
fi

if [ ! -f $PANDORA_HOME/pandora_agent.conf ]
then
	echo " "
	echo "FATAL ERROR: Cannot load pandora_agent.conf"
	echo " "
	exit -1
fi

# Default values
CHECKSUM_MODE=1
DEBUG_MODE=0

echo "$TIMESTAMP - Reading general config parameters from .conf file" >> $PANDORA_HOME/pandora.log
for a in `cat $PANDORA_HOME/pandora_agent.conf | grep -v -e "^#" | grep -v -e "^module" `
do
        a=`echo $a | tr -s " " " "`

        # Get general configuration parameters from config file
        if [ ! -z "`echo $a | grep -e '^server_ip'`" ]
        then
                SERVER_IP=`echo $a | awk '{ print $2 }' `
                echo "$TIMESTAMP - [SETUP] - Server IP Address is $SERVER_IP" >> $PANDORA_HOME/pandora.log
        fi
        if [ ! -z "`echo $a | grep -e '^server_path'`" ]
        then
                SERVER_PATH=`echo $a | awk '{ print $2 }' `
                echo "$TIMESTAMP - [SETUP] - Server Path is $SERVER_PATH" >> $PANDORA_HOME/pandora.log
        fi
        if [ ! -z "`echo $a | grep -e '^temporal'`" ]
        then
                TEMP=`echo $a | awk '{ print $2 }' `
                echo "$TIMESTAMP - [SETUP] - Temporal Path is $TEMP" >> $PANDORA_HOME/pandora.log
        fi
        if [ ! -z "`echo $a | grep -e '^interval'`" ]
        then
                INTERVAL=`echo $a | awk '{ print $2 }' `
                echo "$TIMESTAMP - [SETUP] - Interval is $INTERVAL seconds" >> $PANDORA_HOME/pandora.log
        fi
	if [ ! -z "`echo $a | grep -e '^agent_name'`" ]
        then
              	NOMBRE_HOST=`echo $a | awk '{ print $2 }' `
	        echo "$TIMESTAMP - [SETUP] - Agent name is $NOMBRE_HOST " >> $PANDORA_HOME/pandora.log
        fi
	if [ ! -z "`echo $a | grep -e '^debug'`" ]
        then
              	DEBUG_MODE=`echo $a | awk '{ print $2 }' `
	        echo "$TIMESTAMP - [SETUP] - Debug mode is $DEBUG_MODE " >> $PANDORA_HOME/pandora.log
        fi
	if [ ! -z "`echo $a | grep -e '^checksum'`" ]
        then
              	CHECKSUM_MODE=`echo $a | awk '{ print $2 }' `
	        echo "$TIMESTAMP - [SETUP] - Checksum is $CHECKSUM_MODE " >> $PANDORA_HOME/pandora.log
        fi

done


# MAIN Program loop begin

# OS Data
OS_VERSION=`uname -r`
OS_NAME=`uname -s`
# Hostname
if [ -z "$NOMBRE_HOST" ] 
then 
	NOMBRE_HOST=`/bin/hostname`
fi

while [ "1" == "1" ]
do

	# Fecha y hora. Se genera un serial (numero de segundos desde 1970) para cada paquete generado.
	TIMESTAMP=`date +"%Y/%m/%d %H:%M:%S"`
	SERIAL=`date +"%s"`
	
	# Nombre de los archivos
	DATA=$TEMP/$NOMBRE_HOST.$SERIAL.data
	CHECKSUM=$TEMP/$NOMBRE_HOST.$SERIAL.checksum
	PANDORA_FILES="$TEMP/$NOMBRE_HOST.$SERIAL.*"
	
	# Makes data packet
	echo "<agent_data os_name='$OS_NAME' os_version='$OS_VERSION' interval='$INTERVAL' version='$AGENT_VERSION' timestamp='$TIMESTAMP' agent_name='$NOMBRE_HOST'>" > $DATA
	if [ "$DEBUG_MODE" == "1" ]
	then
		echo "$TIMESTAMP - Reading module adquisition data from .conf file" >> $PANDORA_HOME/pandora.log
	fi
	for a in `cat $PANDORA_HOME/pandora_agent.conf | grep -v -e "^#" | grep -e "^module" ` 
	do
		a=`echo $a | tr -s " " " "`
	
	        if [ ! -z "`echo $a | grep -e '^module_exec'`" ]
	        then
	           execution=`echo $a | cut -c 13- `
	           res=`eval $execution`
	           if [ -z "$flux_string" ]
	           then
	           	res=`eval expr $res`
		   fi
	           echo "<data>$res</data>" >> $DATA
	        fi
	
	        if [ ! -z "`echo $a | grep -e '^module_name'`" ]
	        then
	           name=`echo $a | cut -c 13- `
		   echo "<name>$name</name>" >> $DATA
	        fi
	
	        if [ ! -z "`echo $a | grep -e '^module_begin'`" ]
	        then
	           echo "<module>" >> $DATA
	        fi

		if [ ! -z "`echo $a | grep -e '^module_max' `" ]
		then
		   max=`echo $a | awk '{ print $2 }' `
		   echo "<max>$max</max>" >> $DATA
		fi
		if [ ! -z "`echo $a | grep -e '^module_min'`" ]
		then
		   min=`echo $a | awk '{ print $2 }' `
		   echo "<min>$min</min>" >> $DATA
		fi
		if [ ! -z "`echo $a | grep -e '^module_description'`" ]
		then
		   desc=`echo $a | cut -c 20- `
		   echo "<description>$desc</description>" >> $DATA
		fi
		
	        if [ ! -z "`echo $a | grep -e '^module_end'`" ]
	        then
	           echo "</module>" >> $DATA
	        fi
	
	        if [ ! -z "`echo $a | grep -e '^module_type'`" ]
	        then
	           mtype=`echo $a | awk '{ print $2 }' `
	           if [ ! -z "`echo $mtype | grep 'generic_data_string'`" ]
		   then
			flux_string=1
		   else
	                flux_string=0
	       	        unset flux_string
	           fi
	           echo "<type>$mtype</type>" >> $DATA
        	fi
	done

	# Call for user-defined script for data adquisition

	if [ -f "$PANDORA_HOME/pandora_user.conf" ]
	then
	   /bin/bash $PANDORA_HOME/pandora_user.conf >> $DATA
	fi

	# Finish data packet
	echo "</agent_data>" >> $DATA
	if [ "$DEBUG_MODE" == "1" ]
	then
		echo "$TIMESTAMP - Finish writing XML $DATA" >> $PANDORA_HOME/pandora.log
	fi
	
	if [ "$CHECKSUM_MODE" == "1" ]
	then
		# Calculate Checksum and prepare MD5 file
		CHECKSUM_DATA=`/usr/bin/md5sum $DATA`
        	echo $CHECKSUM_DATA > $CHECKSUM 
	else
		CHECKSUM_DATA="No valid checksum"
		echo $CHECKSUM_DATA > $CHECKSUM
	fi
	
	# Send packets to server and detele it
	scp $PANDORA_FILES pandora@$SERVER_IP:$SERVER_PATH > /dev/null
        if [ "$DEBUG_MODE" == "1" ]
	then
		echo "$TIMESTAMP - Copying $PANDORA_FILES to $SERVER_IP:$SERVER_PATH" >> $PANDORA_HOME/pandora.log
		echo "Debug mode. Agent terminated"
		exit
	fi
	
	rm -f $PANDORA_FILES> /dev/null
	sleep $INTERVAL
done 
# forever! 
