#!/usr/bin/perl

##################################################################################
# Pandora Server
##################################################################################
# Copyright (c) 2004-2005 Sancho Lerena, slerena@gmail.com
# Permission is granted to copy, distribute and/or modify this document
# under the terms of the GNU Free Documentation License, Version 2.0
# or any later version published by the Free Software Foundation at www.gnu.org
##################################################################################

# Includes list
use strict;
use XML::Simple;                	# Useful XML functions
use Digest::MD5;                	# MD5 generation
use Time::Local;                	# DateTime basic manipulation
use DBI;                               	# DB interface with MySQL
use Date::Manip;                	# Needed to manipulate DateTime formats of input, output and compare
use File::Copy;                         # Needed to manipulate files

# version: Defines actual version of Pandora Server
my $version = "1.1";
my $build="050512";

# Setup variables
my $dirname="";
my $dbhost ='';
my $dbuser ='';
my $verbosity ='1'; # Verbose 1 by default
my $dbpass='';
my $server_threshold='';
my $log_file="";
my $pandora_path="";
my $pandora_master=1; # on by default
my $pandora_check=1; # on by default

# FLUSH in each IO
$| = 1;

# Inicio del bucle principal de programa
pandora_init();
# Read config file for Global variables
pandora_loadconfig ($pandora_path);
# Audit server starting
pandora_audit ("Pandora Daemon starting", "SYSTEM", "System");
# Begin pandora_server
pandora_main();


##############################################################################
# Main loop
##############################################################################

sub pandora_main {
	my $config;
	my $file_data;
	my $file_md5;
	my @file_list;
	my $onefile;
	my $agent_filename;

	# Abrimos el directorio de datos y leemos cada fichero
	logger ($log_file,"Launching Pandora Server v $version",0);
	while ( "1" == "1" ) { # Pandora Server Main Loop !
 		opendir(DIR, $dirname) or die "can't opendir $dirname: $!";
 		while (defined($onefile = readdir(DIR))){
   			push @file_list,$onefile; 	# Push in a stack all directory entries for this loop
 		}
 		# KeepAlive checks for Agents, only for master servers
		if ($pandora_master == "1"){
 			keep_alive_check();
		}
        	while (defined($onefile = pop @file_list)) {	# Begin to process files
                	$file_data = "$dirname/$onefile";
                	next if $onefile =~ /^\.\.?$/;     # Skip . and .. directory
                	if ( $onefile =~ /([\-\:\;\.\,\_\s\a\*\=\(\)\/a-zA-Z0-9]*).data/ ) {  # First filter any file that doesnt like ".data"
   				$agent_filename = $1;
   				$file_md5 = "$dirname/$agent_filename.checksum";
				if (( -e $file_md5 ) or ($pandora_check == 0)){ # If check is disabled, ignore if file_md5 exists
    					# Comprobamos integridad
    					my $check_result;
					$check_result = md5check ($file_data,$file_md5);
					if (($pandora_check == 0) || ($check_result == 1)){
                                 		# Procesa_Datos its the main function to process datafile
                                        	# But first we needed to verify integrity of data file
                                        	if ($pandora_check == 1){
							logger ($log_file,"VERBOSE: Integrity of Datafile using MD5 is verified: $file_data",2);
						}

     						eval { # XML Processing error catching procedure. Critical due XML was no validated
                                  			logger ($log_file, "DEBUG: Ready to parse $file_data",4);
                                  			$config = XMLin($file_data, forcearray=>'module');
      							eval { # Gestion de errores en funcion de insercion de datos.
                                                 		procesa_datos($config);
                                         		};
                                         		if ($@) {
                                                 		logger ($log_file, "ERROR: Processing data from XML file $file_data",0);
                                         		}
                          			};
                          			if ($@) {
                                   			logger ($log_file, "ERROR: Error processing XML contents in $file_data",0);
                                   			copy ($file_data,$file_data."_BAD");
                                   			if (($pandora_check == 1) && ( -e $file_md5 )) {
								copy ($file_md5,$file_md5."_BAD");
							}
                          			}
                                        	# If everything its ok..
						# delete files
                                        	unlink ($file_data);
                                        	if ( -e $file_md5 ) {
							unlink ($file_md5);
						}
                                	} else { # md5 check fails
     						logger ($log_file, "ERROR: MD5 Checksum failed! for $file_data",0);
						# delete files
                                        	unlink ($file_data);
                                        	if ( -e $file_md5 ) {
							unlink ($file_md5);
						}
						
    					}
   				} # No existe fichero de checksum, ignoramos el archivo
                	}
        	}
        	closedir(DIR);
        	sleep $server_threshold;
	}
} # End of main loop function

#################################################################################
## SUB pandora_event (evento, id_grupo, id_agente)
## Write in internal audit system an entry.
##################################################################################
sub pandora_event {
        my $evento = $_[0];
        my $id_grupo = $_[1];
        my $id_agente = $_[2];
        my $timestamp = &UnixDate("today","%Y-%m-%d %H:%M:%S");
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306", $dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
        $evento = $dbh->quote($evento);
	my $query = "insert into tevento (id_agente, id_grupo, evento, timestamp, estado) VALUES ($id_agente,$id_grupo,$evento,'$timestamp',0)";
        $dbh->do($query);
        $dbh->disconnect();
}

#################################################################################
## SUB pandora_audit (description, name, action)
## Write in internal audit system an entry.
##################################################################################
sub pandora_audit {
        my $desc = $_[0];
        my $name = $_[1];
        my $action = $_[2];
        my $timestamp = &UnixDate("today","%Y-%m-%d %H:%M:%S");
        my $query = "insert into tsesion (ID_usuario, IP_origen, accion, fecha, descripcion) values ('SYSTEM','".$name."','".$action."','".$timestamp."','".$desc."')";
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306", $dbuser, $dbpass,{ RaiseError => 1, AutoCommit => 1 });
        $dbh->do($query);
        $dbh->disconnect();
}

#################################################################################
## SUB dame_agente_id (nombre_agente)
## Return agent ID, use "nombre_agente" as name of agent.
##################################################################################
sub dame_agente_id {
        my $nombre_agente = $_[0];
        my $id_agente;my @data;
        # Calculamos el ID del agente haciendo una select por su nombre.
        my $query_idag = "select * from tagente where nombre = '$nombre_agente'";
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306", $dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
        my $s_idag = $dbh->prepare($query_idag);

        $s_idag ->execute;
    	if ($s_idag->rows == 0) {
        	logger ($log_file, "ERROR dame_agente_id(): Cannot find agent called $nombre_agente. Returning -1",1);
        	logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
		$data[0]=-1;
    	} else  {           @data = $s_idag->fetchrow_array();   }
    	$id_agente = $data[0];
    	$s_idag->finish();
        $dbh->disconnect();
        return $id_agente;
}

#################################################################################
## SUB dame_grupo_agente (id_agente) 
## Return id_group of an agent given its id
#################################################################################

sub dame_grupo_agente {
	my $id_agente = $_[0];
	my $id_grupo;
	my @data;
	# Calculamos el ID del agente haciendo una select por su id
        my $query_idag = "select * from tagente where id_agente = $id_agente";
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306", $dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
        my $s_idag = $dbh->prepare($query_idag);

        $s_idag ->execute;
    	if ($s_idag->rows == 0) {
        	logger ($log_file, "ERROR dame_grupo_agente(): Cannot find agent with id $id_agente",1);
        	logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
    	} else  {           @data = $s_idag->fetchrow_array();   }
 	$id_grupo = $data[4];
    	$s_idag->finish();
       	$dbh->disconnect();
       	return $id_grupo;
}

#################################################################################
## SUB dame_comando_alerta (id_alerta)
## Return agent ID, use "nombre_agente" as name of agent.
##################################################################################
sub dame_comando_alerta {
        my $id_alerta = $_[0];
	my @data;
        # Calculamos el ID del agente haciendo una select por su nombre.
        my $query_idag = "select * from talerta where id_alerta = $id_alerta";
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306", $dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
        my $s_idag = $dbh->prepare($query_idag);
	my $comando = "";
        $s_idag ->execute;
    	if ($s_idag->rows == 0) {
        	logger ($log_file, "ERROR dame_comando_alerta(): Cannot find alert $id_alerta",1);
        	logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
    	} else  {           
		@data = $s_idag->fetchrow_array();   
    		$comando = $data[2];
	}
    	$s_idag->finish();
    	$dbh->disconnect();
    	return $comando;
}


#################################################################################
## SUB dame_agente_nombre (id_agente)
## Return agent name, given "id_agente"
##################################################################################
sub dame_agente_nombre {
        my $id_agente = $_[0];
        my $nombre_agente;my @data;
        # Calculamos el ID del agente haciendo una select por su nombre.
        my $query_idag = "select * from tagente where id_agente = '$id_agente'";
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",$dbuser, $dbpass,{ RaiseError => 1, AutoCommit => 1 });
        my $s_idag = $dbh->prepare($query_idag);
        $s_idag ->execute;
    	if ($s_idag->rows == 0) {
        	logger ($log_file, "ERROR dame_agente_nombre(): Cannot find agent with id $id_agente",1);
        	logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
    	} else  {           @data = $s_idag->fetchrow_array();   }
    	$nombre_agente = $data[1];                                       ~
    	$s_idag->finish();
        $dbh->disconnect();
        return $nombre_agente;
}


#################################################################################
## SUB dame_modulo_id (nombre_modulo)
## Return module ID, given "nombre_modulo" as module name
##################################################################################
sub dame_modulo_id {
        my $nombre_modulo = $_[0];
        my $id_modulo; my @data;
        # Calculamos el ID del agente haciendo una select por su nombre.
        my $query_idag = "select * from ttipo_modulo where nombre = '$nombre_modulo'";
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306", $dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
        my $s_idag = $dbh->prepare($query_idag);
        $s_idag ->execute;
    	if ($s_idag->rows == 0) {
        	logger ($log_file, "ERROR dame_modulo_id(): Cannot find module called $nombre_modulo ",1);
        	logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
        	$id_modulo = -1;
    	} else  {    
    		@data = $s_idag->fetchrow_array();
    		$id_modulo = $data[0];
    	}
    	$s_idag->finish();
    	$dbh->disconnect;
    	return $id_modulo;
}


#################################################################################
## SUB dame_agente_modulo_id (id_agente, id_tipomodulo, nombre)
## Return agente_modulo ID, from tabla tagente_modulo, given id_agente, id_tipomodulo and name
##################################################################################
sub dame_agente_modulo_id {
        my $id_agente = $_[0];
        my $id_tipomodulo = $_[1];
        my $nombre = $_[2];
        my $id_agentemodulo;
	my @data;
	
        # Calculamos el ID del agente haciendo una select por su nombre.
        my $query_idag = "select * from tagente_modulo where id_agente = '$id_agente' and id_tipo_modulo = '$id_tipomodulo' and nombre = '$nombre'";
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",$dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
        my $s_idag = $dbh->prepare($query_idag);
        $s_idag ->execute;
    	if ($s_idag->rows == 0) {
        	logger ($log_file, "ERROR dame_agente_modulo_id(): Cannot find agente_modulo called $nombre",2);
        	logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
        	$id_agentemodulo = -1;
    	} else  {    
    		@data = $s_idag->fetchrow_array(); 
    		$id_agentemodulo = $data[0];
    	}
    	$s_idag->finish();
    	$dbh->disconnect;
    	return $id_agentemodulo;
}

#################################################################################
## SUB dame_nombreagente_agentemodulo (id_agente_modulo)
## Return agent name diven id_agente_modulo
##################################################################################
sub dame_nombreagente_agentemodulo {
        my $id_agentemodulo = $_[0];
        my $id_agente; my @data;
        # Calculamos el ID del agente haciendo una select por su nombre.
        my $query_idag = "select * from tagente_modulo where id_agente_modulo = ".$id_agentemodulo;
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",$dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
        my $s_idag = $dbh->prepare($query_idag);
        $s_idag ->execute;
    	if ($s_idag->rows == 0) {
        	logger ($log_file, "ERROR dame_nombreagente_agentemodulo(): Cannot find id_agente_modulo $id_agentemodulo",1);
        	logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
    	} else  {    @data = $s_idag->fetchrow_array(); }
    	$id_agente= $data[1];
    	$s_idag->finish();
    	$dbh->disconnect;
    	my $nombre_agente = dame_agente_nombre ($id_agente);
    	return $nombre_agente;
}

#################################################################################
## SUB dame_nombretipomodulo_idtipomodulo (id_tipo_modulo)
## Return name of moduletype given id_tipo_modulo
##################################################################################
sub dame_nombretipomodulo_idagentemodulo {
	my $id_tipomodulo = $_[0]; my @data;
	
	# Calculamos el ID del agente haciendo una select por su nombre.
	my $query_idag = "select * from ttipo_modulo where id_tipo = ".$id_tipomodulo;
	my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",$dbuser, $dbpass,	{ RaiseError => 1, AutoCommit => 1 });
	my $s_idag = $dbh->prepare($query_idag);
	$s_idag ->execute;
	if ($s_idag->rows == 0) {
		logger ($log_file, "ERROR dame_nombreagente_agentemodulo(): Cannot find module type with ID $id_tipomodulo",1);
		logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
	} else  {    @data = $s_idag->fetchrow_array(); }
	my $tipo= $data[1];
	$s_idag->finish();
	$dbh->disconnect;
	return $tipo;
}

#################################################################################
## SUB dame_learnagente (id_agente)
## Return 1 if agent is in learn mode, 0 if not
##################################################################################
sub dame_learnagente {
        my $id_agente = $_[0]; my @data;
        
        # Calculamos el ID del agente haciendo una select por su nombre.
        my $query_idag = "select * from tagente where id_agente = ".$id_agente;
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",$dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
        my $s_idag = $dbh->prepare($query_idag);
        $s_idag ->execute;
    	if ($s_idag->rows == 0) {
        	logger ($log_file, "ERROR dame_learnagente(): Cannot find agente $id_agente",2);
      		logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
    	} else  {    @data = $s_idag->fetchrow_array(); }
    	my $learn= $data[6];
    	$s_idag->finish();
    	$dbh->disconnect;
        return $learn;
}


#################################################################################
## SUB dame_id_tipo_modulo (id_agente_modulo)
## Return id_tipo of module with id_agente_modulo
##################################################################################
sub dame_id_tipo_modulo {
        my $id_agente_modulo = $_[0];
        my $tipo; my @data;
        # Calculamos el ID del agente haciendo una select por su nombre.
        my $query_idag = "select * from tagente_modulo where id_agente_modulo = ".$id_agente_modulo;
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",$dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
        my $s_idag = $dbh->prepare($query_idag);
        $s_idag ->execute;
    	if ($s_idag->rows == 0) {
        	logger ($log_file, "ERROR dame_id_tipo_modulo(): Cannot find id_agente_modulo $id_agente_modulo",1);
      		logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
    	} else  {    @data = $s_idag->fetchrow_array(); }
    	$tipo= $data[2];
    	$s_idag->finish();
    	$dbh->disconnect;
        return $tipo;
}

#################################################################################
## SUB dame_intervalo (id_agente)
## Return interval for id_agente
##################################################################################
sub dame_intervalo {
        my $id_agente = $_[0];
        my $tipo; my @data;
        # Calculamos el ID del agente haciendo una select por su nombre.
        my $query_idag = "select * from tagente where id_agente = ".$id_agente;
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",$dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
        my $s_idag = $dbh->prepare($query_idag);
        $s_idag ->execute;
    	if ($s_idag->rows == 0) {
        	logger ($log_file, "ERROR dame_intervalo(): Cannot find agente $id_agente",1);
      		logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
    	} else  {    @data = $s_idag->fetchrow_array(); }
    	$tipo= $data[7];
    	$s_idag->finish();
    	$dbh->disconnect;
        return $tipo;
}


#################################################################################
## SUB dame_desactivado (id_agente)
## Return disabled = 1 if disabled, 0 if not disabled
##################################################################################
sub dame_desactivado {
        my $id_agente = $_[0];
        my $tipo; my @data;
        # Calculamos el ID del agente haciendo una select por su nombre.
        my $query_idag = "select * from tagente where id_agente = ".$id_agente;
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",$dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
	my $s_idag = $dbh->prepare($query_idag);
        $s_idag ->execute;
    	if ($s_idag->rows == 0) {
        	logger ($log_file, "ERROR dame_desactivado(): Cannot find agente $id_agente",1);
      	 	 logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
    	} else  {    @data = $s_idag->fetchrow_array(); }
    	my $desactivado= $data[12];
    	$s_idag->finish();
    	$dbh->disconnect;
        return $desactivado;
}

#################################################################################
## SUB dame_ultimo_contacto (id_agente)
## Return last_contact for id_agente
##################################################################################
sub dame_ultimo_contacto {
        my $id_agente = $_[0];
        my $tipo; my @data;
        # Calculamos el ID del agente haciendo una select por su nombre.
        my $query_idag = "select * from tagente where id_agente = ".$id_agente;
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306", $dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
        my $s_idag = $dbh->prepare($query_idag);
        $s_idag ->execute;
    	if ($s_idag->rows == 0) {
        	logger ($log_file, "ERROR dame_ultimo_contacto(): Cannot find agente $id_agente",1);
      	 	logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
    	} else  {    @data = $s_idag->fetchrow_array(); }
    	$tipo= $data[5];
    	$s_idag->finish();
    	$dbh->disconnect;
        return $tipo;
}

#################################################################################
## SUB crea_agente_modulo(nombre_agente, nombre_tipo_modulo, nombre_modulo)
## create an entry in tagente_modulo
##################################################################################
sub crea_agente_modulo {
	my $nombre_agente = $_[0];
	my $tipo_modulo = $_[1];
	my $nombre_modulo = $_[2];
	my $max = $_[3];
	my $min = $_[4];
	my $descripcion = $_[5];
		 
	# Pending: Check MAX/MIN values passed in XML
#	my $max =$_[3];
#	my $min= $_[4];

	my $modulo_id = dame_modulo_id($tipo_modulo);
	my $agente_id = dame_agente_id($nombre_agente);
	
	my $query = "insert into tagente_modulo (id_agente,id_tipo_modulo,nombre,max,min,descripcion) values ($agente_id,$modulo_id,'$nombre_modulo',$max,$min,'AUTO: $descripcion')";
	if (($max eq "") and ($min eq "")) {
		$query = "insert into tagente_modulo (id_agente,id_tipo_modulo,nombre,descripcion) values ($agente_id,$modulo_id,'$nombre_modulo','AUTO: $descripcion')";
	} elsif ($min eq "") {
		$query = "insert into tagente_modulo (id_agente,id_tipo_modulo,nombre,max,descripcion) values ($agente_id,$modulo_id,'$nombre_modulo',$max,'AUTO: $descripcion')";
	} elsif ($min eq "") {
		$query = "insert into tagente_modulo (id_agente,id_tipo_modulo,nombre,min,descripcion) values 	($agente_id,$modulo_id,'$nombre_modulo',$min,'AUTO: $descripcion')";
	}
	logger ($log_file, "DEBUG: Query for autocreate : $query ",3);	
	my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",
                    $dbuser, $dbpass,
                    { RaiseError => 1, AutoCommit => 1 });
    $dbh->do($query);
    $dbh->disconnect();
}

#################################################################################
## SUB DB_ultimo_contacto_agente (timestamp,nombre_agente,os_data, agent_version,interval)
## Update last contact field in Agent Table                 	
##################################################################################

sub DB_ultimo_contacto_agente {
        my $timestamp = $_[0];
        my $time_now = &UnixDate("today","%Y-%m-%d %H:%M:%S");
        my $nombre_agente = $_[1];
        my $os_data = $_[2];
        my $agent_version = $_[3];
        my $interval = $_[4];
        my $id_agente = dame_agente_id($nombre_agente);
        my $query = "";
        if ($interval == -1){ # no update for interval field (some old agents doest support it)
			$query = "update tagente set agent_version = '$agent_version', ultimo_contacto_remoto = '$timestamp', ultimo_contacto = '$time_now', os_version = '$os_data' where id_agente = $id_agente";                	
        } else {
			$query = "update tagente set intervalo = $interval, agent_version = '$agent_version', ultimo_contacto_remoto = '$timestamp', ultimo_contacto = '$time_now', os_version = '$os_data' where id_agente = $id_agente";
        }
        logger ($log_file, "DEBUG: DB_ultimo_contacto_agente: Updating Agent last contact data for $nombre_agente",2);
	logger ($log_file, "DEBUG: DB_ultimo_contacto_agente: SQL Query: ".$query,5);
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306", $dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
        my $sag = $dbh->prepare($query);
        $sag ->execute;
    	$sag ->finish();
        $dbh->disconnect;
}


#################################################################################
## SUB DB_insert_datos (timestamp,nombre_agente,tipo_modulo,nombre_modulo,datos)
## Insert data in main table: tagente_datos
       
##################################################################################

sub DB_insert_datos {
    	my $timestamp = $_[0];
        my $nombre_agente = $_[1];
        my $tipo_modulo = $_[2];
        my $nombre_modulo = $_[3];
        my $datos = $_[4];
        my $max = $_[5];
	my $min = $_[6];
	my $descripcion = $_[7];
	my @data;
	
        # Obtenemos los identificadores
        my $id_agente = dame_agente_id($nombre_agente);
        
        # Check if exists module and agent_module reference in DB, if not, and learn mode activated, insert module in DB
        my $id_modulo = dame_modulo_id($tipo_modulo);
        my $id_agente_modulo = dame_agente_modulo_id($id_agente, $id_modulo, $nombre_modulo);
        my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306", $dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
 
	# take max and min values for this id_agente_module
        if ($id_agente_modulo != -1){
		my $query_idag = "select * from tagente_modulo where id_agente_modulo = ".$id_agente_modulo;
		my $dbh2 = DBI->connect("DBI:mysql:pandora:$dbhost:3306", $dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
		my $s_idag = $dbh2->prepare($query_idag);
		$s_idag ->execute;
		if ($s_idag->rows == 0) {
			logger ($log_file, "ERROR Cannot find agenteModulo $id_agente_modulo",2);
			logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
		} else  {    @data = $s_idag->fetchrow_array(); }
		$max = $data[5];
		$min = $data[6];
		$s_idag->finish();
		$dbh2->disconnect;
	} else {
		if (dame_learnagente($id_agente) == 1){
		# Try to write a module and agent_module definition for that datablock
			logger ($log_file, "VERBOSE: DB_insert_datos MUST CREATE a MODULE IN LEARN MODE FOR agent $nombre_agente",1);
			crea_agente_modulo($nombre_agente, $tipo_modulo, $nombre_modulo, $max, $min, $descripcion);  		
			$id_agente_modulo = dame_agente_modulo_id($id_agente, $id_modulo, $nombre_modulo);
			goto DB_insert_datos_1;
		}
		else {
			logger ($log_file, "VERBOSE: DB_insert_datos cannot find module definition ($nombre_modulo / $tipo_modulo )for agent $nombre_agente - Use LEARN MODE for autocreate.",1);
			goto fin_DB_insert_datos;
		}
	} # Module exists or has been created
DB_insert_datos_1:
        my $query;
	my $outlimit = 0;
	if ( $tipo_modulo eq "generic_data_string" ) {
		$datos = $dbh->quote($datos); # Parse data entry for adecuate SQL representation.
		$query = "insert into tagente_datos_string (id_agente_modulo,datos,timestamp,id_agente) VALUES ($id_agente_modulo,$datos,'$timestamp',$id_agente)";	
	} else {
		if ($max != $min) {
			if ($datos > $max) { 
				$datos = $max; 
				$outlimit=1;
				logger ($log_file, "DEBUG: MAX Value reached ($max) for agent $nombre_agente / $nombre_modulo",2);
			}			
			if ($datos < $min) { 
				$datos = $min;
				$outlimit = 1;
				logger ($log_file, "DEBUG: MIN Value reached ($min) for agent $nombre_agente / $nombre_modulo",2);
			}
		}
		$query = "insert into tagente_datos (id_agente_modulo,datos,timestamp,id_agente) VALUES ($id_agente_modulo,'$datos','$timestamp',$id_agente)";
	} # If data is out of limits, do not insert into database (Thanks for David Villanueva for his words)
	if ($outlimit == 0){       
		logger ($log_file, "DEBUG: DB_insert_datos Calculado id_agente_modulo a $id_agente_modulo",3);
        	logger ($log_file, "DEBUG: DB_insert_datos SQL : $query",4);
        	$dbh->do($query);
	}

fin_DB_insert_datos:
	$dbh->disconnect;
}


#################################################################################
## SUB calcula_alerta  (timestamp,nombre_agente,tipo_modulo,nombre_modulo,datos)
## Dado un modulo de datos, me dice si hay que generar alerta o no
##################################################################################

sub calcula_alerta {
        my $timestamp = $_[0];
        my $nombre_agente = $_[1];
        my $tipo_modulo = $_[2];
        my $nombre_modulo = $_[3];
        my $datos = $_[4];
        my $id_modulo;
	my $id_agente;
        my $id_agente_modulo;
	my $max;
	my $min; # for calculate max & min to generate ALERTS

        # Obtemos los ID's a traves del paquete de datos
        $id_agente = dame_agente_id($nombre_agente);
        $id_modulo = dame_modulo_id($tipo_modulo);
        $id_agente_modulo = dame_agente_modulo_id($id_agente,$id_modulo,$nombre_modulo);

        logger ($log_file, "DEBUG: calcula_alerta() Calculado id_agente_modulo a $id_agente_modulo",3);

        # Buscamos si existe una alerta definida para esta combinacion agente/modulo
    	my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",$dbuser, $dbpass,{ RaiseError => 1, AutoCommit => 1 });
    	my $query_idag = "select * from talerta_agente_modulo where id_agente_modulo = '$id_agente_modulo'";
    	my $s_idag = $dbh->prepare($query_idag);
        $s_idag ->execute;
        my @data;
	# If exists a defined alert for this module then continue
	if ($s_idag->rows != 0) {
		while (@data = $s_idag->fetchrow_array()) {
			my $id_aam = $data[0];
			my $id_alerta = $data[2];
			my $id_agente_modulo = $data[1];
			my $id_agente = dame_agente_id(dame_nombreagente_agentemodulo($id_agente_modulo));
			my $id_grupo = dame_grupo_agente($id_agente);
			my $campo1 = $data[3];
			my $campo2 = $data[4];
			my $campo3 = $data[5];
			my $descripcion = $data[6];
			my $dis_max = $data[7];
			my $dis_min = $data[8];
			my $threshold = $data[9];
			my $last_fired = $data[10];
			my $max_alerts = $data[11];
			my $times_fired = $data[12];
			my $comando ="";
			logger($log_file, "Found an alert defined for $nombre_modulo, its ID $id_alerta",5);
			# Here we process alert if conditions are ok
			# Get data for defined alert given as $id_alerta
			my $query_idag2 = "select * from talerta where id_alerta = '$id_alerta'";
			my $s2_idag = $dbh->prepare($query_idag2);
			$s2_idag ->execute;
			my @data2;
			if ($s2_idag->rows != 0) {
				while (@data2 = $s2_idag->fetchrow_array()) {
					$comando = $data2[2];
				}
			}
			$s2_idag->finish();

			# v1.1alpha 19/02/04, slerena@gmail.com
              		# Get MAX and MIN value for this Alert. Only generate alerts if value is ABOVE MIN and BELOW MAX.
			my @data_max; 
                	my $query_idag_max = "select * from tagente_modulo where id_agente_modulo = ".$id_agente_modulo;
                	my $dbh_max = DBI->connect("DBI:mysql:pandora:$dbhost:3306", $dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
                	my $s_idag_max = $dbh_max->prepare($query_idag_max);
               	 	$s_idag_max ->execute;
                	if ($s_idag_max->rows == 0) {
                        	logger ($log_file, "ERROR Cannot find agenteModulo $id_agente_modulo",2);
                        	logger ($log_file, "ERROR: SQL Query is $query_idag ",2);
                	} else  {    @data = $s_idag_max->fetchrow_array(); }
                	$max = $data_max[5];
                	$min = $data_max[6];
                	$s_idag_max->finish();
                	$dbh_max->disconnect;
	 
			# Alert exec main code
			my $alert_fired = 0;
			if (($max_alerts >= $times_fired) && (($datos > $dis_max) || ($datos < $dis_min))){
				$alert_fired = 1;
                		# Only if between max & min, and max number of fired alerts is above number of already fired alerts
				my $fecha_ultima_alerta = ParseDate($last_fired);
				my $fecha_actual = ParseDate( $timestamp );
				my $ahora_mysql = &UnixDate("today","%Y-%m-%d %H:%M:%S");       # If we need to update MYSQL last_fired will use $ahora_mysql
				my $time_threshold = $threshold;
				my $err; my $flag;
				my $fecha_limite = DateCalc($fecha_ultima_alerta,"+ $time_threshold seconds",\$err);
				$flag = Date_Cmp($fecha_actual,$fecha_limite);
				if ( $flag < 0 ) {
					# Time_Treshold restrict this ALERT, doesnt fire now
					logger ($log_file, "Alert activated, but timer isnt run out yet",1);
				} else { # timer-ok block begin
					# Alert Trigger is ON !!!!!
					my $evt_descripcion = "Alert fired ($nombre_modulo)";
					pandora_event($evt_descripcion,$id_grupo,$id_agente);
            				# Calculation of time threshold
            				my $times_fired = $times_fired + 1;
            				my $query_idag = "update talerta_agente_modulo set times_fired = $times_fired, last_fired = '$ahora_mysql' where id_aam = $id_aam ";
 					$dbh->do($query_idag);
					my $nombre_agente = dame_nombreagente_agentemodulo($id_agente_modulo);
					logger($log_file, "Alert TRIGGERED for $nombre_agente ! ",1);
					# If exists a defined alert for this module then continue
					if ($id_alerta != 0){ # id_alerta 0 is reserved for internal audit system
						$comando =~ s/_field1_/"$campo1"/g;
						$comando =~ s/_field2_/"$campo2"/g;
						$comando =~ s/_field3_/"$campo3"/g;
						$comando =~ s/_agent_/$nombre_agente/g;
						$comando =~ s/_timestamp_/$timestamp/g;
						$comando =~ s/_data_/$datos/g;
						# Clean up some "tricky" characters
						$comando =~ s/&gt;/>/g;
						# EXECUTING COMMAND !!!
						eval {
							my $exit_value = system ($comando);
							$exit_value  = $? >> 8; # Shift 8 bits to get a "classic" errorlevel
							if ($exit_value != 0) {
								logger ($log_file, "Executed command for triggered alert had errors (errorlevel =! 0) ",0);
								logger ($log_file, "Executed command was $comando ",0);
							}
						};
						if ($@){
			                        	logger ($log_file, "ERROR: Error executing alert command  ( $comando )",0);
			                       		logger ($log_file, "ERROR Code: $@",1);
						}
                        		} else { # id_alerta = 0, is a internal system audit
						logger($log_file, "Internal audit lauch for agent name $nombre_agente",2);
						$campo1 =~ s/_timestamp_/$timestamp/g;
						$campo1 =~ s/_data_/$datos/g;
						pandora_audit ($campo1, $nombre_agente, "User Alert");
				        }
				} # Of else timer-ok block
			} # if (($max_alerts >= $times_fired) && (($datos > $dis_max) || ($datos < $dis_min))) block
			elsif (($alert_fired == 0) && ($times_fired != 0)){ # If alert doesnt fired and fired counter isnt zero, lets reset counter
				$query_idag = "update talerta_agente_modulo set times_fired = 0 where id_aam = $id_aam ";
				$dbh->do($query_idag);
			} # end of elsif block of main condition
		} # while (@data = $s_idag->fetchrow_array()) block
         } # If defined alert
	$s_idag->finish();
    	$dbh->disconnect;
}
                
                
#################################################################################
## SUB DB_escribe_estado (nombre_agente,tipo_modulo,nombre_modulo,valor_datos, estado)
## Modifica los datos, cambiando el estado del los modulos en la tabla de estados.
##################################################################################

sub DB_escribe_estado {
	# my $timestamp = $_[0];
	# slerena, 05/10/04 : Fixed bug because differences between agent / server time source.
	# now we use only local timestamp to stamp state of modules
	my $timestamp = &UnixDate("today","%Y-%m-%d %H:%M:%S");
	my $nombre_agente = $_[0];
	my $tipo_modulo = $_[1];
	my $nombre_modulo = $_[2];
	my $datos = $_[3]; # OJO, no pasa una estructura sino un valor discreto
	my $estado = $_[4];
	my @data;
	my $cambio = 0; my $id_grupo; 
	
	# Obtenemos los identificadores
	# OJO; no controlamos si los valores pasados como cadena son validos.
	# PENDIENTE: Comprobacion de errores aqui.
	
	my $id_agente = dame_agente_id($nombre_agente);
	my $id_modulo = dame_modulo_id($tipo_modulo);
	my $id_agente_modulo = dame_agente_modulo_id($id_agente, $id_modulo, $nombre_modulo);
	
	logger ($log_file, "DB_escribe_estado Calculado id_agente_modulo a $id_agente_modulo",3);

	# Check alert subroutine
	eval { # Hacemos la comprobacion de las alertas aqui. OJO !
		# Alerts checks for Agents, only for master servers
                if ($pandora_master == "1"){
			calcula_alerta($timestamp, $nombre_agente, $tipo_modulo, $nombre_modulo, $datos, $estado);
		}
	};
	if ($@) {
			logger ($log_file, "ERROR: Error in SUB calcula_alerta(). ModuleName: $nombre_modulo ModuleType: $tipo_modulo AgentName: $nombre_agente",1);
			logger ($log_file, "ERROR Code: $@",1)
	}
	my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",$dbuser, $dbpass, { RaiseError => 1, AutoCommit => 1 });
	# $id_agente apunta al ID del agente que queremos actualizar".dame_nombreagente_agentemodulo ($id_agente_modulo)."
	# Vamos a ver si existe una entrada en la tabla  tagente_estado
	my $idages = "select * from tagente_estado where id_agente_modulo = $id_agente_modulo";
	my $s_idages = $dbh->prepare($idages);
	$s_idages ->execute;
	$datos = $dbh->quote($datos);
	my $query_act; # OJO que dentro de una llave solo tiene existencia en esa llave !!
	if ($s_idages->rows == 0) {            # No hay entrada, hay que hacer la primera con un INSERT
		logger ($log_file, "Generando entrada (INSERT) en tagente_estado para $nombre_modulo",2);
    		$query_act = "insert into tagente_estado (id_agente_modulo,datos,timestamp,estado,cambio) values ($id_agente_modulo,$datos,'$timestamp','$estado','1')"; # Cuando se hace un insert, siempre hay un cambio de estado
    	} else {
           	 # Hacemos un update porque ya existe una entrada
	        @data = $s_idages->fetchrow_array();
	        # Se supone que $data[5] nos daria el estado ANTERIOR, podriamos leerlo
	        # ($c1,$c2,$c3...) $i_dages->fetchrow_array(); y luego hacer referencia a $c6 p.e
	        if ( $data[5] != $estado){
	                # Cambio de estado detectado !
	                $cambio = 1;
	                # Este seria el momento oportuno de probar a saltar la alerta si estuviera definida
			# Makes an event entry
			$id_grupo = dame_grupo_agente($id_agente);
			my $descripcion;
 			if ( $estado == 0) { $descripcion = "Monitor ($nombre_modulo) goes up "; }
			if ( $estado == 1) { $descripcion = "Monitor ($nombre_modulo) goes down"; }
			pandora_event($descripcion,$id_grupo,$id_agente);
	        }
    		$query_act = "update tagente_estado set datos = $datos, cambio = '$cambio', timestamp = '$timestamp', estado = '$estado' where id_agente_modulo = $id_agente_modulo ";  		
    	}
	my $a_idages = $dbh->prepare($query_act);
    	logger ($log_file, "DEBUG: DB_escribe_estado SQL [ $query_act ]",4);
	$a_idages->execute;
	$a_idages->finish();
    	$s_idages->finish();
	$dbh->disconnect;
}


#################################################################################
## SUB keep_alive_check  ()
## Calculate a global keep alive check for agents without data and alert defined 
##################################################################################

sub keep_alive_check {
        # Buscamos si existe una alerta definida para esta combinacion agente/modulo
    	my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",$dbuser, $dbpass,{ RaiseError => 1, AutoCommit => 1 });
    	my $query_idag = "select * from talerta_agente_modulo";
    	my $s_idag = $dbh->prepare($query_idag);
        $s_idag ->execute;
        my @data; my $err; my $flag;
	
	if ($s_idag->rows != 0) {
	while (@data = $s_idag->fetchrow_array()) {
		my $id_aam = $data[0];
		my $id_alerta = $data[2];
		my $id_agente_modulo = $data[1];
		# Only checks keep_alive special modules (-1 on type)
		if (dame_id_tipo_modulo($id_agente_modulo) == -1) {
			my $campo1 = $data[3];
			my $campo2 = $data[4];
			my $campo3 = $data[5];
			my $dis_max = $data[7];
			my $dis_min = $data[8];
			my $threshold = $data[9];
			my $last_fired = $data[10];
			my $max_alerts = $data[11];
			my $times_fired = $data[12];
			my $alert_fired = 0;
			my $fecha_ultima_alerta = ParseDate($last_fired);
			my $ahora_mysql = &UnixDate("today","%Y-%m-%d %H:%M:%S");
			my $timestamp = $ahora_mysql;
			my $fecha_actual = ParseDate( $ahora_mysql );
			# If we need to update MYSQL last_fired will use $ahora_mysql

			# Calculate if INTERVAL x2 for this agent is bigger than sub last contact date with actual date
			my $nombre_agente = dame_nombreagente_agentemodulo($id_agente_modulo);
			my $id_agente = dame_agente_id($nombre_agente);
			if (dame_desactivado($id_agente) == 0){
				my $fecha_ultimocontacto = dame_ultimo_contacto($id_agente);
				my $intervalo = dame_intervalo($id_agente); # Seconds
				my $intervalo_2 = $intervalo*2;
				$fecha_ultimocontacto = ParseDate($fecha_ultimocontacto);
				my $fecha_limite = DateCalc($fecha_ultimocontacto,"+ $intervalo_2 seconds",\$err);
				$flag = Date_Cmp($fecha_actual,$fecha_limite);
				if ( $flag >= 0) { $alert_fired = 1 } else { $alert_fired=0 } ;
				if (( $flag >= 0 ) && ($max_alerts >= $times_fired)){ # Calculate if max_alerts for this time is exhausted
					# Alert Trigger is ON !
					# Check if alert is fired by event-success
					my $time_threshold = $threshold; # from defined alert 
					$fecha_limite = DateCalc($fecha_ultima_alerta,"+ $time_threshold seconds",\$err);
					$flag = Date_Cmp($fecha_actual,$fecha_limite);
					if ( $flag >= 0 ) {
						# Alert Trigger is fired by time-threshold
						# Get "command" string from Alert Definition in DB
						my $comando = dame_comando_alerta($id_alerta);
						$times_fired = $times_fired + 1;
						$query_idag = "update talerta_agente_modulo set times_fired = $times_fired, last_fired = '$ahora_mysql' where id_aam = $id_aam ";
						my $s3_idag = $dbh->prepare($query_idag);
						$s3_idag ->execute;
						$s3_idag->finish();
						my $nombre_agente = dame_nombreagente_agentemodulo($id_agente_modulo);
						logger($log_file, "Alert (KeepAlive) TRIGGERED for $nombre_agente ! ",1);
						my $id_grupo = dame_grupo_agente($id_agente);
						my $descripcion = "Agent down";
						pandora_event($descripcion,$id_grupo,$id_agente);
						if ($id_alerta > 0){ # id_alerta 0 is reserved for internal audit system
							$comando =~ s/_field1_/"$campo1"/g;
							$comando =~ s/_field2_/"$campo2"/g;
							$comando =~ s/_field3_/"$campo3"/g;
							$comando =~ s/_agent_/$nombre_agente/g;
							$comando =~ s/_timestamp_/$timestamp/g;
							# Clean up some "tricky" characters
							$comando =~ s/&gt;/>/g;
							# EXECUTING COMMAND !!!
							eval {
								my $exit_value = system ($comando);
								$exit_value  = $? >> 8; # Shift 8 bits to get a "classic" errorlevel
								if ($exit_value != 0) {
									logger ($log_file, "Executed command for triggered alert had errors (errorlevel =! 0) ",0);
								}
							};
							if ($@){
								logger ($log_file, "ERROR: Error executing alert command  ( $comando )",1);
								logger ($log_file, "ERROR Code: $@",2);
							}
						} else { # id_alerta = 0, is a internal system audit
								logger($log_file, "Internal audit lauch for agent name $nombre_agente",2);
								$campo1 =~ s/_timestamp_/$timestamp/g;
								pandora_audit ($campo1, $nombre_agente, "User Alert");
						}
					} # if $flag >=0 (for time-firing calculation)
					} elsif (($alert_fired == 0) && ($times_fired != 0)){ # If alert doesnt fired and fired counter isnt zero, lets reset counter
						$query_idag = "update talerta_agente_modulo set times_fired = 0 where id_aam = $id_aam ";
						my $s3_idag = $dbh->prepare($query_idag);
						$s3_idag ->execute;
						$s3_idag ->finish();
						my $id_grupo = dame_grupo_agente($id_agente);
						my $descripcion = "Agent up";
						pandora_event($descripcion,$id_grupo,$id_agente);
					} # if $flag >= 0 (for time-threshold)
				} # Disabled agent
			} #if (dame_id_tipo_modulo($id_agente_modulo) == -1)
		} # While
	} # if ($s_idag->rows != 0) 
	$s_idag->finish();
    	$dbh->disconnect;
}

#################################################################################
## SUB procesa_datos (par1)
## Procesa un paquete de datos (XML preprocesado)
##################################################################################
## param_1 : Nombre de la estructura contenedora de datos (XML)

sub procesa_datos {
    my $datos = $_[0]; my $tipo_modulo; my $agent_name; my $timestamp; my $interval; my $os_version; my $agent_version;
    eval {
		$agent_name = $datos->{'agent_name'};
		$timestamp = $datos->{'timestamp'};
		$agent_version = $datos->{'version'};
		$interval = $datos->{'interval'};
		$os_version = $datos->{'os_version'};
		# Check for parameteres, not all version agentes gives the same parameters ! 
		if (length($interval) == 0){ $interval = -1; } # No update for interval !      
		if (length($os_version) == 0){ $os_version = "N/A"; } # No update for interval ! 
		
		logger($log_file,"Interval was $interval for $agent_name",2);
		logger($log_file,"OS Data was $os_version for $agent_name",2);    		              
		
		DB_ultimo_contacto_agente($timestamp, $agent_name, $os_version, $agent_version, $interval);
		foreach my $part(@{$datos->{module}}) {
			$tipo_modulo = $part->{type}->[0];
			logger($log_file,"Proccessing packet type [ $tipo_modulo ][ $agent_name ]",3);
	
			if ($tipo_modulo eq 'generic_data') {
				module_generic_data($part,$timestamp,$agent_name);
			}
			elsif ($tipo_modulo eq 'generic_data_inc') {
				module_generic_data_inc($part,$timestamp,$agent_name);
			}
			elsif ($tipo_modulo eq 'generic_data_string') {
				module_generic_data_string($part,$timestamp,$agent_name);
			}
			elsif ($tipo_modulo eq 'generic_proc') {
				module_generic_proc($part,$timestamp,$agent_name);
			}
			else {
				logger ($log_file,"ERROR: Ha llegado un modulo desconocido ($tipo_modulo)",1);
			}
		}
	}; # eval
	if ($@) {
		logger ($log_file,"ERROR: Error dentro de la funcion de proceso de datos, for Agent ".$datos->{'agent_name'},1);
		logger ($log_file,"ERROR Code: $@",1);
	}
}


#################################################################################
####   MODULOS implementados en Pandora
#################################################################################

# ----------------------------------------+
# Modulos genericos de Pandora            |
# ----------------------------------------+

# Los modulos genericos de pandora son de 4 tipos
#
# generic_data . Almacena numeros enteros largos, util para monitorizar proceos que
#                                general valores o sensores que devuelven valores.

# generic_proc . Almacena informacion booleana (cierto/false), util para monitorizar
#                 procesos logicos.

# generic_data_inc . Almacena datos igual que generic_data pero tiene una logica
#                                que sirve para las fuentes de datos que alimentan el agente con datos
#                                que se incrementan continuamente, por ejemplo, los contadores de valores
#                                en las MIB de los adaptadores de red, las entradas de cierto tipo en
#                                un log o el nº de segundos que ha pasado desde X momento. Cuando el valor
#                                es mejor que el anterior o es 0, se gestiona adecuadamente el cambio.

# generic_data_string. Store a string, max 255 chars.

#################################################################################
## SUB module_generic_proc (param_1, param_2, param_3)
## Procesa datos genericos sobre un proceso
##################################################################################
## param_1 : Nombre de la estructura contenedora de datos (XML)
## paran_2 : Timestamp del paquete de datos
## param_3 : Agent name

sub module_generic_proc {
	my $datos = $_[0];
	my $a_timestamp = $_[1];
	my $agent_name = $_[2];
	my $estado;
	# Leemos datos de la estructura
	my $a_datos = $datos->{data}->[0];
	my $a_name = $datos->{name}->[0];
	my $a_desc = $datos->{description}->[0];
	my $a_max = $datos->{max}->[0];
	my $a_min = $datos->{min}->[0];
	if (ref($a_max) eq "HASH") {
        	$a_max = "";
        }
	if (ref($a_min) eq "HASH") {
		$a_min = "";
	}
	if (ref($a_datos) eq "HASH") {
		logger ($log_file,"ERROR: Bad data value receibed in $a_name module",0);
	} else {

		DB_insert_datos($a_timestamp,$agent_name,"generic_proc",$a_name,$a_datos,$a_max,$a_min,$a_desc);
	
		# Calculamos su estado
		if ( $datos->{'data'}->[0] < 1 ) { $estado = 1 } else { $estado=0 };
	
		# Escribimos su estado
		DB_escribe_estado ($agent_name,"generic_proc",$a_name,$a_datos,$estado);
	}
}


#################################################################################
## SUB module_generic_data (par1, par2)
## Procesa datos generados por Modulo de adquisicion de datos numericos
##################################################################################
## param_1 : Nombre de la estructura contenedora de datos (XML)
## paran_2 : Timestamp del paquete de datos
## param_3 : Nombre del agente

sub module_generic_data {
	my $datos = $_[0];
	my $m_timestamp = $_[1];
	my $agent_name = $_[2];

	# Leemos datos de la estructura
	my $m_name = $datos->{name}->[0];
	my $a_desc = $datos->{description}->[0];
	my $m_data = $datos->{data}->[0];
	
	my $a_max = $datos->{max}->[0];
	my $a_min = $datos->{min}->[0];

	if (ref($a_max) eq "HASH") {
                $a_max = "";
        }
        if (ref($a_min) eq "HASH") {
                $a_min = "";
        }
        if (ref($m_data) eq "HASH") {
                logger ($log_file,"ERROR: Bad data value receibed in $m_name module",0);
        } else {

		DB_insert_datos($m_timestamp,$agent_name,"generic_data",$m_name,$m_data,$a_max,$a_min,$a_desc);
	
		# Calculamos su estado (su estado siempre es bueno, jeje)
		my $estado = 100; # N/A
	
		# Escribimos su estado
		DB_escribe_estado ($agent_name,"generic_data",$m_name,$m_data,$estado);
	}
}

#################################################################################
## SUB module_generic_data_inc (par1, par2)
## Procesa datos generados por Modulo de adquisicion de datos numericos incrementales
##################################################################################
## param_1 : Nombre de la estructura contenedora de datos (XML)
## paran_2 : Timestamp del paquete de datos
## param_3 : Nombre del agente

sub module_generic_data_inc {
	my $datos = $_[0];
	my $m_timestamp = $_[1];
	my $agent_name = $_[2];
	
	# Leemos datos de la estructura
	my $m_name = $datos->{name}->[0];
	my $a_desc = $datos->{description}->[0];
	my $m_data = $datos->{data}->[0];
	my $a_max = $datos->{max}->[0];
    	my $a_min = $datos->{min}->[0];
	if (ref($a_max) eq "HASH") {
                $a_max = "";
        }
        if (ref($a_min) eq "HASH") {
                $a_min = "";
        }
        if (ref($m_data) eq "HASH") {
                logger ($log_file,"ERROR: Bad data value receibed in $m_name module",0);
        } else {

	my $no_existe=0;
	my $timestamp = &UnixDate("today","%Y-%m-%d %H:%M:%S");
  	
	# Algoritmo:
	# 1) Buscamos el valor anterior en la base de datos
	# 2) Si el dato nuevo es mayor o igual, guardamos en la tabla de datos general la diferencia y en la tabla de estado de datos incrementales, modificamos el valor por el actual.
	# 3) Si el dato nuevo es menor, guardamos el valor completo en la tabla de datos general y en la tabla de estado de datos incrementales, modificamos el valor por el actual.
	
	# Luego:
	# a) Obtener valor anterior, si no existe, el valor anterior sera 0
	# b) Comparar ambos valores (anterior y actual)
	# c) Actualizar tabla de estados de valores incrementales
	# d) Insertar valor en tabla de valores de datos generales
	
	# Obtemos los ID's a traves del paquete de datos
	my $id_agente = dame_agente_id($agent_name);
	my $id_modulo = dame_modulo_id("generic_data_inc");
	my $id_agente_modulo = dame_agente_modulo_id($id_agente,$id_modulo,$m_name);
	my $query_idag = "select * from tagente_datos_inc where id_agente_modulo = $id_agente_modulo";
	my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",$dbuser, $dbpass,{ RaiseError => 1, AutoCommit => 1 });
	my $s_idag = $dbh->prepare($query_idag);
	my $diferencia; my @data_row; my $data_anterior;
	$s_idag ->execute;
	if ($s_idag->rows == 0) {
		$diferencia = 0;
		$no_existe = 1;
	} else {
		@data_row = $s_idag->fetchrow_array();
		$data_anterior = $data_row[2];
		$diferencia = $m_data - $data_anterior;
	}
	$s_idag->finish();
	$dbh->disconnect;
	
	# c) Actualizar tabla de estados de valores incrementales (se pone siempre el ultimo valor)
	
	# Primero comprobamos si existe ! y si no, creamos
	if ($no_existe == 1){
		my $query = "insert into tagente_datos_inc (id_agente_modulo,datos,timestamp) VALUES ($id_agente_modulo,'$m_data','$timestamp')";
		my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",$dbuser, $dbpass,	{ RaiseError => 1, AutoCommit => 1 });
		$dbh->do($query);
		$dbh->disconnect;
	} else { # Si existe, modificamos
		my $query_idag = "update tagente_datos_inc set datos = '$m_data' where id_agente_modulo  = $id_agente_modulo";
		my $dbh = DBI->connect("DBI:mysql:pandora:$dbhost:3306",$dbuser, $dbpass,	{ RaiseError => 1, AutoCommit => 1 });
		$s_idag = $dbh->prepare($query_idag);
		$s_idag ->execute;
		$s_idag->finish();
		$dbh->disconnect;
	}
	
	my $nuevo_data = 0; # Bug reported 18/10/04: a problem with data_inc in state table, storing physical value in data field, not incremental data
	if ($diferencia >= 0) {
		if ($no_existe==0) {
			$nuevo_data = $diferencia;
		}
	} else { # Si diferencia = 0 o menor (problemilla?)
		if ($no_existe !=0){
			# Houston, we have a Problem !
			logger ($log_file,"ERROR: Error inside data_inc algorimth, for Agent $agent_name and Type Generic_data_inc ",0);
		}
	}
	DB_insert_datos($m_timestamp,$agent_name,"generic_data_inc",$m_name,$nuevo_data);
	
	# Calculamos su estado (su estado siempre es bueno, jeje)
	my $estado = 100; # N/A
	# Escribimos su estado
	DB_escribe_estado ($agent_name,"generic_data_inc",$m_name,$nuevo_data,$estado);
	}
}

#################################################################################
## SUB module_generic_data_string (par1, par2)
## Procesa datos generados por el modulo de adquisicion de datos alfanumericos
##################################################################################
## param_1 : Nombre de la estructura contenedora de datos (XML)
## paran_2 : Timestamp del paquete de datos
## param_3 : Nombre del agente

sub module_generic_data_string {
	my $datos = $_[0];
	my $m_timestamp = $_[1];
	my $agent_name = $_[2];
	
	# Leemos datos de la estructura
	my $m_name = $datos->{name}->[0];
	my $m_data = $datos->{data}->[0];
	my $a_desc = $datos->{description}->[0];
	my $a_max = $datos->{max}->[0];
        my $a_min = $datos->{min}->[0];
	if (ref($a_max) eq "HASH") {
                $a_max = "";
        }
        if (ref($a_min) eq "HASH") {
                $a_min = "";
        }
	# Obtemos los ID's a traves del paquete de datos
	my $id_agente = dame_agente_id($agent_name);
	my $id_modulo = dame_modulo_id("generic_data_string");
	my $id_agente_modulo = dame_agente_modulo_id($id_agente,$id_modulo,$m_name);
	
	DB_insert_datos($m_timestamp,$agent_name,"generic_data_string",$m_name,$m_data,$a_max,$a_min,$a_desc);

    	# Su estado va a ser N/A.
    	my $estado = 100; # N/A
	DB_escribe_estado ($agent_name,"generic_data_string",$m_name,$m_data,$estado);
}

# ----------------------------------------+
# Otras funciones generales de Pandora    |
# ----------------------------------------+

#################################################################################
# SUB md5check (par1, par2)
# Comprobacion MD5 del archivo
#################################################################################
# param_1 : Nombre de archivo datos
# param_2 : Nombre de archivo con MD5

sub md5check {
	my $buf;
	my $buf2;
	my $file = $_[0];
	my $md5file = $_[1];
	open(FILE, $file) or return 0;
	binmode(FILE);
	my $md5 = Digest::MD5->new;
	while (<FILE>) {
		$md5->add($_);
	}
	close(FILE);
	$buf2 = $md5->hexdigest;
	open(FILE,$md5file) or return 0;
	while (<FILE>) {
		$buf = $_;
	}
	close (FILE);
	$buf=uc($buf);
	$buf2=uc($buf2);
	if ($buf =~ /$buf2/ ) {
		#print "MD5 Correct";
		return 1;
	} else {
		#print "MD5 Incorrect";
		return 0;
	}
}

#################################################################################
# SUB logger (par1, par2)
# Vuelca informacion a un archivo (para hacer log)
#################################################################################
# param_1 : Nombre de archivo datos
# param_2 : Datos

sub logger {
	my $fichero = $_[0];
	my $datos = $_[1];
	my $verbose_level = $_[2];
	
	if ($verbose_level <= $verbosity) {
		if ($verbose_level == 1) {      $datos = "(VERBOSE) " . $datos; }
		if ($verbose_level == 2) {      $datos = "(DEBUG) " . $datos; }
		my $time_now = &UnixDate("today","%Y/%m/%d %H:%M:%S");
		open (FILE, ">> $fichero") or die "Error abriendo el fichero log";
		select(FILE);
		$| = 1;
		print FILE "$time_now $datos \n";
		close (FILE);
	}
}

##############################################################################
# SUB help_screen()
#  Show a help screen an exits
##############################################################################

sub help_screen{
	printf "\nSintax: \n   pandora_server.pl  fullpathname_to_pandora_server.conf \n\n";
	printf "Following options are optional : \n";
	printf "            -v  :  Verbose mode activated, give more information in logfile \n";
	printf "            -d  :  Debug mode activated, give extensive information in logfile \n";
	printf "            -h  : This screen, show a little help screen \n";
	printf " \n";
	exit;
}

##############################################################################
# SUB pandora_init ()
# Makes the initial parameter parsing, initializing and error checking
##############################################################################

sub pandora_init {
	printf "\nPandora Server  $version Build $build, Copyright (c) 2004-2005 <slerena\@gmail.com>\n";
    	printf "Permission is granted to copy, distribute and/or modify this document \n";
    	printf "under the terms of the GNU Free Documentation License, Version 2.0 \n";
    	printf "or any later version published by the Free Software Foundation at www.gnu.org \n\n\n";

	# Load config file from command line
	if ($#ARGV == -1 ){
		print "I Need at least one parameter: Complete path to pandora_server.conf \n";
		help_screen;
		exit;
	}
   
	# If there are not valid parameters
	my $parametro;
	my $ltotal=$#ARGV; my $ax;
	for ($ax=0;$ax<=$ltotal;$ax++){
		$parametro = $ARGV[$ax];
		if ($parametro =~ m/-h\z/i ) { help_screen();  }
			elsif ($parametro =~ m/-help\z/i ) { help_screen();  }
			elsif ($parametro =~ m/--help\z/i ) { help_screen();  }
		elsif ($parametro =~ m/-v\z/i) { $verbosity=5; }
		elsif ($parametro =~ m/-d\z/i) { $verbosity=10; }
		else { ($pandora_path = $parametro); }
	}
	if ($pandora_path eq ""){
		print "FATAL ERROR: I Need complete path to pandora_server.conf file !!\n\n";
		exit;
	}
}


##############################################################################
# Read external configuration file
##############################################################################

sub pandora_loadconfig {
	my $archivo_cfg = @_[0];
	my $buffer_line;
	my @command_line;
	# Check for file
	if ( ! -e $archivo_cfg ) {
		printf "[ERROR] Cannot open configuration file. Please specify a valid one in command line \n";
		exit 1;
	}

	# Collect items from config file and put in an array 
	open (CFG, "< $archivo_cfg");
	while (<CFG>){
		$buffer_line = $_;
		if ($buffer_line =~ m/([\w-_\.]+)\s([0-9\w-_\.\/\?\&\=\)\(\_\-\!\*\@\#\%\$\~\"\']+)/){
			push @command_line,$1;
			push @command_line,$2;
		}
	}
  
 	close (CFG);
 	# Process this array with commandline like options 
 	# Process input parameters
 	my @args = @command_line;
 	my $parametro;
 	my $ltotal=$#args; my $ax;

 	# Has read setup file ok ?
 	if ( $ltotal == 0 ) {
  		print "[ERROR] No valid setup tokens readed in $archivo_cfg ";
  		exit;
 	}
 
 	for ($ax=0;$ax<=$ltotal;$ax++){
  		$parametro = $args[$ax];
  		if ($parametro =~ m/dirname\z/) {  $dirname = $args[$ax+1]; $ax++; } 
  		elsif ($parametro =~ m/dbuser\z/) { $dbuser  = $args[$ax+1]; $ax++; } 
  		elsif ($parametro =~ m/dbpass\z/) { $dbpass = $args[$ax+1]; $ax++; } 
  		elsif ($parametro =~ m/dbhost\z/) { $dbhost  = $args[$ax+1]; $ax++; } 
  		elsif ($parametro =~ m/log_file\z/) { $log_file = $args[$ax+1]; $ax++; }
		elsif ($parametro =~ m/checksum\z/) { $pandora_check = $args[$ax+1]; $ax++; }
		elsif ($parametro =~ m/master\z/) { $pandora_master = $args[$ax+1]; $ax++; }
  		elsif ($parametro =~ m/verbosity\z/) { $verbosity = $args[$ax+1]; $ax++; } 
  		elsif ($parametro =~ m/server_threshold\z/) { $server_threshold = $args[$ax+1]; $ax++; } 
 	}
 
 	# Check for valid token token values
 	if (( $dbuser eq "" ) || ( $dirname eq "" ) || ( $log_file eq "" ) || ( $dbhost eq "")  || ( $pandora_master eq "") || ( $dbpass eq "" ) ) {
  		print "[ERROR] Bad Config values. Be sure that $archivo_cfg is a valid setup file";
  		exit;
 	}
  	printf "Pandora Server now initialized and running ... \n\n";
}

